/*
 * Tato trida komunikuje s vyssi vrstvou pomoci TCP socketu na portu 10007.
 * Po uvodnim "dialogu", ve kterem se prijmou typy Nodu, nody samotne a
 * adresa subsystemu se vytvori dve vlakna. Prvni vlakno ceka v blokujicim
 * rezimu na prijem dat a druhe ceka na prijem dat od planovace.
 */

#ifndef CONNECTOR_H_
#define CONNECTOR_H_

#include "ConditionQueue.h"
#include "DeviceType.h"
#include "Node.h"
#include "TCPSocket.h"

#include "json/reader.h"
#include "json/writer.h"

#include <list>
#include <string>
#include <unordered_map>

typedef std::unordered_map<unsigned short, Node> nodeMap;

class Delegator;

class Connector {
public:
	Connector();
	virtual ~Connector();

	/*
	 * Metoda pro predani objektu TCPSocket, ktery prijima a odesila pakety
	 */
	void setTCPSocket(TCPSocket);

	/*
	 * Metoda, ktera predava referenci delegatoru.
	 */
	void setDelegator(Delegator *dlg);

	/*
	 * Metoda, ktera spousti komunikaci s vyssi vrstvou. Musi se zavolat
	 * az pro predani TCPSocketu a delegatoru.
	 */
	void start();

	void updateNodeData(Node &);

	void requestNewAddresses(nodeMap &nodes);

	void logError(unsigned char);

	void logError(unsigned char, unsigned short);

	void addNodeToMap(Node &node);


	/*
	 * konstanty, ktere identifikuji typ prichozich a odchozich zprav
	 */
	static const unsigned int REQUEST_SUB_ADDRESS = 1; //pozadavek o zaslani adresy subsystemu
	static const unsigned int RESPONSE_SUB_ADDRESS = 2; //odpoved na pozadavek o zaslani adresy subsystemu

	static const unsigned int DISCOVER_NEW_NODES = 15;

	static const unsigned int NEW_NODES_FOUND = 3; //vracime, pokud jsme nasli nove nody

	static const unsigned int NEW_ADDRESSES = 4; // server posila nove adresy nalezenych nodu
	static const unsigned int DATA_GET = 5;
	static const unsigned int DATA_SET = 6;

	static const unsigned int ERROR = 7;

	static const unsigned int REQUEST_NODE_LIST = 8;
	static const unsigned int RESPONSE_NODE_LIST = 9;

	static const unsigned int REGISTER_NEW_TYPE = 10;
	static const unsigned int NODE_REMOVED = 11;

	static const unsigned int REQUEST_TYPE_LIST = 12;
	static const unsigned int RESPONSE_TYPE_LIST = 13;
	static const unsigned int PING_REQUEST = 16;
	static const unsigned int PING_RESPONSE = 17;


private:
	void getAddress();
	void getTypes();
	void getNodes();

	void receiveThread();
	void sendThread();

	void setData(Node &, Json::Value &);

	unsigned char address;
	int messageNumber;

	ConditionQueue<std::string> messageQueue;

	TCPSocket socket;
	std::string clearChars (std::string);

	Json::Value messageSend;
	Json::Value messageReceive;
	Json::Value nodeData;
	Json::Value array;
	Json::Reader reader;
	Json::FastWriter writer;
	std::string receive;

	unsigned int **receiveArray;

	std::unordered_map <unsigned short, Node> nodes;
	std::unordered_map <unsigned short, Node> newNodes;

	std::unordered_map <int, DeviceType> types;

	bool parsed;
protected:
	Delegator *pDelegator;
};

#endif
