#include "Message.h"

#include "Connector.h"

#include <iostream>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

using namespace std;

/* Public methods: */

Message::Message() {
}

Message::~Message() {
}

int Message::makeDiscovery(unsigned char *txbuf) {
	txbuf[0] = START_CHAR;
	txbuf[1] = 1; //lenght
	txbuf[2] = MESSAGE_DISCOVERY; //type
	txbuf[3] = calculateChecksum(txbuf, 1, 2);
	txbuf[4] = END_CHAR;

	return 5;
}

int Message::makeDiscoveryAddress(unsigned char *txbuf, Node node) {
	unsigned char addressHigher = (unsigned char) (node.getAddress() >> 8);
	unsigned char addressLower = (unsigned char) (node.getAddress() & 0xff);
	unsigned char randomHigher = (unsigned char) (node.getRandom() >> 8);
	unsigned char randomLower = (unsigned char) (node.getRandom() & 0xff);

	txbuf[0] = START_CHAR;
	txbuf[1] = 6; //lenght
	txbuf[2] = MESSAGE_DISCOVERY_ADDRESS; //type
	txbuf[3] = randomHigher;
	txbuf[4] = randomLower;
	txbuf[5] = addressHigher;
	txbuf[6] = addressLower;
	txbuf[7] = address;
	txbuf[8] = calculateChecksum(txbuf, 1, 7);
	txbuf[9] = END_CHAR;

	return 10;
}

int Message::makeDiscoveryCheckOk(unsigned char *txbuf, Node node) {
	unsigned char addressHigher = (unsigned char) (node.getAddress() >> 8);
	unsigned char addressLower = (unsigned char) (node.getAddress() & 0xff);
	unsigned char randomHigher = (unsigned char) (node.getRandom() / 256);
	unsigned char randomLower = (unsigned char) (node.getRandom() % 256);

	txbuf[0] = START_CHAR;
	txbuf[1] = 6; //lenght
	txbuf[2] = MESSAGE_DISCOVERY_CHECK_OK; //type
	txbuf[3] = randomHigher;
	txbuf[4] = randomLower;
	txbuf[5] = addressHigher;
	txbuf[6] = addressLower;
	txbuf[7] = address;
	txbuf[8] = calculateChecksum(txbuf, 1, 7);
	txbuf[9] = END_CHAR;

	return 10;
}

int Message::makeMeassure(unsigned char* txbuf, Node node) {
	unsigned char addressHigher = (unsigned char) (node.getAddress() >> 8);
	unsigned char addressLower = (unsigned char) (node.getAddress() & 0xff);

	txbuf[0] = START_CHAR;
	txbuf[1] = 4; //lenght
	txbuf[2] = MESSAGE_MEASURE; //type
	txbuf[3] = addressHigher;
	txbuf[4] = addressLower;
	txbuf[5] = address;
	txbuf[6] = calculateChecksum(txbuf, 1, 5);
	txbuf[7] = END_CHAR;

	return 8;
}

int Message::makeRequest(unsigned char* txbuf, Node node) {
	unsigned char addressHigher = (unsigned char) (node.getAddress() >> 8);
	unsigned char addressLower = (unsigned char) (node.getAddress() & 0xff);

	txbuf[0] = START_CHAR;
	txbuf[1] = 4; //lenght
	txbuf[2] = MESSAGE_REQUEST; //type
	txbuf[3] = addressHigher;
	txbuf[4] = addressLower;
	txbuf[5] = address;
	txbuf[6] = calculateChecksum(txbuf, 1, 5);
	txbuf[7] = END_CHAR;

	return 8;
}

int Message::makeSet(unsigned char* txbuf, Node node) {
	unsigned char addressHigher = (unsigned char) (node.getAddress() >> 8);
	unsigned char addressLower = (unsigned char) (node.getAddress() & 0xff);

	txbuf[0] = START_CHAR;
	txbuf[1] = 4 + node.getDataLength(); //lenght
	txbuf[2] = MESSAGE_SET; //type
	txbuf[3] = addressHigher;
	txbuf[4] = addressLower;
	txbuf[5] = address;

	unsigned int **data = node.getData();

	int channelCount = node.getChannelCount();	//informace o poctu kanalu nodu
	int dataCount = node.getType().getDataCount(); //informace o poctu dat
	int dataRange = node.getType().getDataRange(); //informace o sirce dat

	unsigned char byteValue = 0; //zde se ulozi hodnota dat pro aktualni kanal a pozici
	unsigned char byteIndex = 0; //index, ktery ukazuje pozici v novem "komprimovanem poli"
	unsigned char bitValue = 0; //hodnota aktualniho bitu
	unsigned char bitIndex = 0; //index aktualniho bitu v novem poli
	unsigned int currentData = 0;

	for (unsigned char channel = 0; channel < channelCount; channel++) {
		for (unsigned char count = 0; count < dataCount; count++) {
			currentData = data[channel][count];

			for (int range = 0; range < dataRange; range++) {
				bitValue = ((currentData >> range) & 0x01);

				if (bitValue == 0) {
					byteValue &= ~(0x01 << bitIndex);
				} else {
					byteValue |= 0x01 << bitIndex;
				}

				txbuf[byteIndex + 6] = byteValue;
				bitIndex++;
				if (bitIndex == 8) {
					bitIndex = 0;
					byteIndex++;
					byteValue = 0;
				}
			}

		}

	}

	txbuf[7 + byteIndex] = calculateChecksum(txbuf, 1, (6 + byteIndex));
	txbuf[8 + byteIndex] = END_CHAR;

	return 9 + byteIndex;
}

void Message::setAddress(unsigned char address) {
	this->address = address;
}

int Message::get(unsigned char* rxbuf, Node node, int n) {
	int change = -1;
	unsigned char crc8 = 0;
	unsigned char length = node.getDataLength();
	unsigned int ** data;

	for (int i = 0; i < n; i++) {
		crc8 = calculateChecksum(rxbuf, i + 1, i + length + 5);
		if (rxbuf[i] == START_CHAR && rxbuf[i + 1] == (4+length)
				&& rxbuf[i + 2] == MESSAGE_GET
				&& rxbuf[i + length + 7] == END_CHAR
				&& rxbuf[i + length + 6] == crc8) {
			change = 0;
			unsigned char channelCount = node.getChannelCount(); //informace o poctu kanalu nodu
			unsigned char dataCount = node.getType().getDataCount(); //informace o poctu dat
			unsigned char dataRange = node.getType().getDataRange(); //informace o sirce dat
			unsigned int intValue = 0; //nova data

			unsigned char intIndex = 0; //index, ktery ukazuje pozici v novem "komprimovanem poli"
			unsigned char bitValue = 0; //hodnota aktualniho bitu
			unsigned char bitIndex = 0; //index aktualniho bitu v novem poli
			unsigned char byteIndex = 0;
			unsigned char byteValue;

			data = node.getData();
			byteValue = rxbuf[i+6];

			for (unsigned char channel = 0; channel < channelCount; channel++) {
				for (unsigned char count = 0; count < dataCount; count++) {
					intValue = 0;
					intIndex = 0;

					for (unsigned char range = 0; range < dataRange; range++) {
						bitValue = ((byteValue >> bitIndex) & 0x01);

						if (bitValue == 0) {
							intValue &= ~(0x01 << intIndex);
						} else {
							intValue |= 0x01 << intIndex;
						}
						intIndex++;

						bitIndex++;
						if (bitIndex == 8) {
							bitIndex = 0;
							byteIndex++;
							byteValue = rxbuf[i + 6 + byteIndex];

						}
					}
					if (data[channel][count] != intValue) {
						change = 1;
					}

					data[channel][count] = intValue;

				}
			}
		}
	}
	if (change == 1) {
		node.setData(data);
	}

	return change;
}

void Message::discoveryResponse(unsigned char *rxbuf, int n, nodeMap &nodes,
		typeMap &types) {
	int random;
	int channelCount;
	int deviceType;

	unsigned char crc8 = 0;

	for (int i = 0; i < n; i++) {
		crc8 = calculateChecksum(rxbuf, i + 1, i + 6);
		if (rxbuf[i] == START_CHAR && rxbuf[i + 1] == 5
				&& rxbuf[i + 2] == MESSAGE_DISCOVERY_RESPONSE
				&& rxbuf[i + 8] == END_CHAR && rxbuf[i + 7] == crc8) {
			Node node;
			random = rxbuf[i + 5] * 256 + rxbuf[i + 6];
			channelCount = rxbuf[i + 3];
			deviceType = rxbuf[i + 4];

			node.setRandom(random);
			node.setChannelCount(channelCount);
			node.setType(types[deviceType]);
			cout << "Mame tu nod s randomem " << random << " a typem"
					<< deviceType << endl;

			nodes.insert(make_pair(node.getRandom(), node));
			i = i + 8;
		}
	}

}

bool Message::discoveryCheck(unsigned char *rxbuf, int n, Node node) {
	unsigned int random;
	unsigned short address;

	unsigned char crc8 = 0;

	for (int i = 0; i < n; i++) {
		crc8 = calculateChecksum(rxbuf, i + 1, i + 7);
		if (rxbuf[i] == START_CHAR && rxbuf[i + 1] == 6
				&& rxbuf[i + 2] == MESSAGE_DISCOVERY_CHECK
				&& rxbuf[i + 9] == END_CHAR && rxbuf[i + 8] == crc8) {
			random = rxbuf[i + 3] * 256 + rxbuf[i + 4];
			address = rxbuf[i + 5] * 256 + rxbuf[i + 6];
			if (node.getAddress() == address && node.getRandom() == random) {
				return true;
			}
		}
	}

	return false;
}

/* Private methods: */

unsigned char Message::calculateChecksum(unsigned char* t, short startIndex,
		short stopIndex) {
	short index;
	unsigned char crcValue = 0;
	for (index = startIndex; index <= stopIndex; index++) {
		crcValue = crcTab[(crcValue ^ t[index])]; //xors the crcvalue with given array and uses it as an index for crcTab to gather a new crcValue
	}
	return crcValue;
}
