#include "UART.h"

#include <fcntl.h>
#include <limits.h>
#include <stdio.h>
#include <sys/ioctl.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <termios.h>
#include <unistd.h>
#include <string.h>
#include <iostream>

using namespace std;

UART::UART() {
	port = 0;
	error = 0;
}

UART::UART(string comportName, int baudrate) {
	this->baudrate = baudrate;
	this->portname = comportName.c_str();
	port = 0;
	error = 0;
}

int UART::openUART(string comportName, int baudrate) {
	this->baudrate = baudrate;
	this->portname = comportName.c_str();
	port = 0;
	error = 0;
	return openUART();
}

UART::~UART() {
}

int UART::openUART() {
	int baudr, status;

	switch (baudrate) {
	case 50:
		baudr = B50;
		break;
	case 75:
		baudr = B75;
		break;
	case 110:
		baudr = B110;
		break;
	case 134:
		baudr = B134;
		break;
	case 150:
		baudr = B150;
		break;
	case 200:
		baudr = B200;
		break;
	case 300:
		baudr = B300;
		break;
	case 600:
		baudr = B600;
		break;
	case 1200:
		baudr = B1200;
		break;
	case 1800:
		baudr = B1800;
		break;
	case 2400:
		baudr = B2400;
		break;
	case 4800:
		baudr = B4800;
		break;
	case 9600:
		baudr = B9600;
		break;
	case 19200:
		baudr = B19200;
		break;
	case 38400:
		baudr = B38400;
		break;
	case 57600:
		baudr = B57600;
		break;
	case 115200:
		baudr = B115200;
		break;
	case 230400:
		baudr = B230400;
		break;
	case 460800:
		baudr = B460800;
		break;
	case 500000:
		baudr = B500000;
		break;
	case 576000:
		baudr = B576000;
		break;
	case 921600:
		baudr = B921600;
		break;
	case 1000000:
		baudr = B1000000;
		break;
	default:
		printf("invalid baudrate\n");
		return (1);
		break;
	}

	port = open(portname, O_RDWR | O_NOCTTY | O_NDELAY);
	if (port == -1) {
		perror("unable to open comport ");
		return (1);
	}

	memset(&portSettings, 0, sizeof(portSettings)); /* clear the new struct */

	portSettings.c_cflag = baudr | CS8 | CLOCAL | CREAD;
	portSettings.c_iflag = IGNPAR;
	portSettings.c_oflag = 0;
	portSettings.c_lflag = 0;
	error = tcsetattr(port, TCSANOW, &portSettings);
	if (error == -1) {
		close(port);
		perror("unable to adjust portsettings ");
		return (1);
	}

	if (ioctl(port, TIOCMGET, &status) == -1) {
		perror("unable to get portstatus");
		return (1);
	}

	status |= TIOCM_DTR; /* turn on DTR */
	status |= TIOCM_RTS; /* turn on RTS */

	if (ioctl(port, TIOCMSET, &status) == -1) {
		perror("unable to set portstatus");
		return (1);
	}

	return (0);
}

int UART::pollComport(unsigned char *buf, int size) {
	int n;

	n = read(port, buf, size);
	return (n);
}

void UART::flush() {
	tcflush(port, TCIOFLUSH);
}

int UART::sendByte(unsigned char byte) {
	int n;

	n = write(port, &byte, 1);
	if (n < 0)
		return (1);

	return (0);
}

unsigned char rxbufpart[128];

int UART::sendBuf(unsigned char *buf, int size) {
	write(port, buf, size);

	int n;
	int rxpos = 0;
	for (int i = 0; i < 8000; i++) {
		usleep(5);
		n = read(port, rxbufpart, 128);
		rxpos = rxpos + n;
		if (rxpos == size) break;
	}

/*	for (int i = 0; i < size; i++) {
		if (buf[i] == rxbuf[i]) {
			cout << "Bajt " << i << " sedi" << endl;
		}
	}
*/
	return (1);
}

void UART::closeComport() {
	int status;

	if (ioctl(port, TIOCMGET, &status) == -1) {
		perror("unable to get portstatus");
	}

	status &= ~TIOCM_DTR; /* turn off DTR */
	status &= ~TIOCM_RTS; /* turn off RTS */

	if (ioctl(port, TIOCMSET, &status) == -1) {
		perror("unable to set portstatus");
	}

	close(port);
}

/*
 Constant  Description
 TIOCM_LE  DSR (data set ready/line enable)
 TIOCM_DTR DTR (data terminal ready)
 TIOCM_RTS RTS (request to send)
 TIOCM_ST  Secondary TXD (transmit)
 TIOCM_SR  Secondary RXD (receive)
 TIOCM_CTS CTS (clear to send)
 TIOCM_CAR DCD (data carrier detect)
 TIOCM_CD  Synonym for TIOCM_CAR
 TIOCM_RNG RNG (ring)
 TIOCM_RI  Synonym for TIOCM_RNG
 TIOCM_DSR DSR (data set ready)

 http://linux.die.net/man/4/tty_ioctl
 */

int UART::isDCDEnabled() {
	int status;

	ioctl(port, TIOCMGET, &status);

	if (status & TIOCM_CAR)
		return (1);
	else
		return (0);
}

int UART::isCTSEnabled() {
	int status;

	ioctl(port, TIOCMGET, &status);

	if (status & TIOCM_CTS)
		return (1);
	else
		return (0);
}

int UART::isDSREnabled() {
	int status;

	ioctl(port, TIOCMGET, &status);

	if (status & TIOCM_DSR)
		return (1);
	else
		return (0);
}

void UART::enableDTR() {
	int status;

	if (ioctl(port, TIOCMGET, &status) == -1) {
		;
		perror("unable to get portstatus");
	}

	status |= TIOCM_DTR; /* turn on DTR */

	if (ioctl(port, TIOCMSET, &status) == -1) {
		perror("unable to set portstatus");
	}
}

void UART::disableDTR() {
	int status;

	if (ioctl(port, TIOCMGET, &status) == -1) {
		perror("unable to get portstatus");
	}

	status &= ~TIOCM_DTR; /* turn off DTR */

	if (ioctl(port, TIOCMSET, &status) == -1) {
		perror("unable to set portstatus");
	}
}

void UART::enableRTS() {
	int status;

	if (ioctl(port, TIOCMGET, &status) == -1) {
		perror("unable to get portstatus");
	}

	status |= TIOCM_RTS; /* turn on RTS */

	if (ioctl(port, TIOCMSET, &status) == -1) {
		perror("unable to set portstatus");
	}
}

void UART::disableRTS() {
	int status;

	if (ioctl(port, TIOCMGET, &status) == -1) {
		perror("unable to get portstatus");
	}

	status &= ~TIOCM_RTS; /* turn off RTS */

	if (ioctl(port, TIOCMSET, &status) == -1) {
		perror("unable to set portstatus");
	}
}

void UART::cputs(const char *text) /* sends a string to serial port */
{
	while (*text != 0)
		sendByte(*(text++));
}
