#include "Connector.h"

#include "Delegator.h"
#include "DeviceType.h"
#include "Errors.h"
#include "Node.h"

#include <unistd.h>

#include <iostream>
#include <thread>


using namespace std;

/* Public methods: */

Connector::Connector() {
}

Connector::~Connector() {
}

void Connector::start() {
	getAddress(); //ziskame adresu subsystemu
	getTypes(); //ziskame seznam existujich typu
	getNodes(); //ziskame seznam existujicich zarizeni

	std::thread sendT([=] {sendThread();});
	sendT.detach();
	pDelegator->startPlanners();
	receiveThread();
}

void Connector::setDelegator(Delegator *dlg) {
	this->pDelegator = dlg;
}

void Connector::setTCPSocket(TCPSocket socket) {
	this->socket = socket;
}

void Connector::logError(unsigned char errorNumber) {
	messageSend.clear();
	messageSend["message_type"] = ERROR;
	messageSend["error"] = errorNumber;

	messageQueue.push(writer.write(messageSend));
}

void Connector::logError(unsigned char errorNumber,
		unsigned short nodeAddress) {
	messageSend.clear();
	messageSend["message_type"] = ERROR;
	messageSend["error"] = errorNumber;
	messageSend["node_address"] = nodeAddress;

	messageQueue.push(writer.write(messageSend));
}

void Connector::updateNodeData(Node& node) {
	messageSend.clear();
	messageSend["message_type"] = DATA_GET;
	messageSend["address"] = node.getAddress();

	int channelCount = node.getChannelCount();
	int dataCount = node.getType().getDataCount();

	Json::Value arr;
	for (int channel = 0; channel < channelCount; channel++) {

		Json::Value chann;
		Json::Value data;
		for (int count = 0; count < dataCount; count++) {
			data.append(node.getData()[channel][count]);
		}
		chann["channel"] = channel;
		chann["data"] = data;
		arr.append(chann);
	}
	messageSend["data"] = arr;

	messageQueue.push(writer.write(messageSend));
}

void Connector::requestNewAddresses(nodeMap& nodes) {
	messageSend.clear();
	messageSend["message_type"] = NEW_NODES_FOUND;
	Json::Value nodeArr;

	for (nodeMap::iterator iter = nodes.begin(); iter != nodes.end(); iter++) {
		Json::Value nodeJson;
		Node node = iter->second;
		nodeJson["random"] = node.getRandom();
		nodeJson["type"] = node.getType().getType();
		nodeJson["channel_count"] = node.getChannelCount();
		nodeArr.append(nodeJson);
	}

	messageSend["nodes"] = nodeArr;

	messageQueue.push(writer.write(messageSend));
}

void Connector::addNodeToMap(Node &node) {
	nodes.insert(make_pair(node.getAddress(), node));
}

/* Private methods: */

void Connector::sendThread() {
	string msg;
	while (true) {
		msg = messageQueue.pop();
		socket.sendData(msg);
	}
}

void Connector::receiveThread() {
	while (true) {
		//prijmeme adresu
		receive = socket.receiveData();
		receive = clearChars(receive);
		messageReceive.clear();
		reader.parse(receive, messageReceive, false);
		messageNumber = messageReceive.get("message_type", "").asInt();

		if (messageNumber == DATA_SET) {
			Node node = nodes[messageReceive.get("address", "").asInt()];
			nodeData = messageReceive["data"];
			setData(node, nodeData);

			pDelegator->setNodeData(node);
		}

		if (messageNumber == REGISTER_NEW_TYPE) {
			DeviceType type;

			type.setType(messageReceive.get("type", "").asInt());
			type.setDataRange(messageReceive.get("data_range", "").asInt());
			type.setDataCount(messageReceive.get("data_count", "").asInt());
			type.setReadWrite(messageReceive.get("read_write", "").asInt());
			type.setLatency(messageReceive.get("latency", "").asInt());

			types.insert(make_pair(type.getType(), type));
		}

		if (messageNumber == DISCOVER_NEW_NODES) {
			cout << "Prijat discovery packet" << endl;
			newNodes.clear();
			pDelegator->discoverNewNodes();

		}

		if (messageNumber == NEW_ADDRESSES) {
			array = messageReceive["nodes"];
			Json::Value a;
			for (unsigned int i = 0; i < array.size(); i++) {
				Node node;

				a = array[i];
				node.setAddress(a.get("address", "").asInt());
				node.setWired(true);
				node.setRandom(a.get("random", "").asInt());
				node.setChannelCount(a.get("channel_count", "").asInt());
				node.setWired(true);
				node.setType(types[a.get("type", "").asInt()]);

				node.makeData();
				newNodes.insert(make_pair(node.getAddress(), node));
			}
			pDelegator->addressNewNodes(newNodes);

		}

		if (messageNumber == NODE_REMOVED) {
			unsigned short address = messageReceive["address"].asUInt();
			Node node = nodes[address];
			cout << "Remove node with id " << node.getAddress() << " and type" << node.getType().getType() << endl;
			pDelegator->removeNode(node);
			nodes.erase(node.getAddress());
		}

		if (messageNumber == PING_REQUEST) {
			messageSend.clear();
			messageSend["message_type"] = PING_RESPONSE;
			messageQueue.push(writer.write(messageSend));
		}

	}
}

void Connector::setData(Node &node, Json::Value &nodeData) {
	receiveArray = node.getData();
	unsigned int channel;
	unsigned int data;

	//cyklus pro kazdy kanal nodu
	for (unsigned char index = 0; index < node.getChannelCount(); index++) {
		channel = nodeData[index]["channel"].asUInt();

		//cyklus pro pocet dat nodu
		for (unsigned char dataIndex = 0; dataIndex < node.getType().getDataCount();
				dataIndex++) {
			data = nodeData[index]["data"][dataIndex].asUInt();
			receiveArray[channel][dataIndex] = data;
		}

	}

	node.setData(receiveArray);
}

void Connector::getAddress() {
	//posleme pozadavek o poslani adresy
	messageSend["message_type"] = REQUEST_SUB_ADDRESS;
	cout << "Zadost o adresu " + writer.write(messageSend) << endl;
	socket.sendData(writer.write(messageSend)); //chceme adresu

	//prijmeme adresu
	receive = socket.receiveData();
	receive = clearChars(receive);
	messageReceive.clear();
	parsed = reader.parse(receive, messageReceive, false);
	if (parsed) {
		address = messageReceive.get("address", "0").asInt();
		pDelegator->setAddress(address);
	} else {
		logError(ERROR_JSON_NOT_VALID);
	}
}

void Connector::getTypes() {
	//posleme pozadavek na seznam typu
	messageSend.clear();
	messageSend["message_type"] = REQUEST_TYPE_LIST;
	socket.sendData(writer.write(messageSend));

	//prijmeme seznam typu
	receive = socket.receiveData();
	receive = clearChars(receive);

	messageReceive.clear();
	parsed = reader.parse(receive, messageReceive, false);
	if (parsed) {
		Json::Value array = messageReceive["types"];
		Json::Value a;
		for (unsigned int i = 0; i < array.size(); i++) {
			DeviceType type;

			a = array[i];
			type.setType(a.get("type", "").asInt());
			type.setDataRange(a.get("data_range", "").asInt());
			type.setDataCount(a.get("data_count", "").asInt());
			type.setReadWrite(a.get("read_write", "").asInt());
			type.setLatency(a.get("latency", "").asInt());

			types.insert(make_pair(type.getType(), type));
			pDelegator->addType(type);
		}
	} else {
		logError(ERROR_JSON_NOT_VALID);
	}
}

void Connector::getNodes() {
	//posleme pozadavek seznamu zarizeni
	messageSend.clear();
	messageSend["message_type"] = REQUEST_NODE_LIST;
	socket.sendData(writer.write(messageSend));

	//prijmeme seznam zarizeni, pokud nejake existuji
	receive = socket.receiveData();
	receive = clearChars(receive);

	messageReceive.clear();
	parsed = reader.parse(receive, messageReceive, false);
	if (parsed) {
	Json::Value array = messageReceive["nodes"];
	Json::Value a;
	for (unsigned int i = 0; i < array.size(); i++) {
		Node node;

		a = array[i];
		nodeData = a["data"];
		node.setAddress(a.get("address", "").asInt());
		node.setChannelCount(a.get("channel_count", "").asInt());
		node.setWired(true);
		node.setType(types[a.get("type", "").asInt()]);

		node.makeData();
		setData(node, nodeData);
		nodes.insert(make_pair(node.getAddress(), node));

		pDelegator->addNode(node);
	}
	} else {
		logError(ERROR_PACKET_NONSENSE);
	}
}

string Connector::clearChars(string allocator) {
	string n = "";
	for (char& c : allocator) {
		if ((int) c >= 32 && (int) c <= 126)
			n = n + c;
	}

	return n;
}
