#ifndef PLANNER_H_
#define PLANNER_H_

#include "Message.h"
#include "GPIO.h"
#include "UART.h"

#include <sys/time.h>

#include <string>
#include <vector>
#include <unordered_map>

typedef std::unordered_map<unsigned short, Node> nodeMap;
typedef std::unordered_map<int, DeviceType> typeMap;

static const int TIMEOUT_US = 10000; //maximum time

class Delegator;

class Planner {
public:
	Planner(std::string uartPort, std::string gpioNum);
	virtual ~Planner();
	void setDelegator(Delegator *dlg);
	void setAddress(unsigned char address);
	void start();
	void addNode(Node &node);
	void addType(DeviceType &type);
	void setNodeData(Node &node);
	void removeNode(Node &);
	void discoverNewNodes();
	void addressNewNodes(nodeMap &newNodes);

private:
    struct timeval t1, t2;
    long long t;

    UART uart;

    std::string uartPort;
    std::string gpioNum;

	Message message;
	unsigned char rxbuf[128];
	unsigned char rxbufpart[128];
	unsigned char txbuf[128];
	unsigned char address;
	int txsize;
	int pass;
	int i, n;

	int **data;

	void loop();

	void requestGetNodes(nodeMap &nodes);
	void setNodes(nodeMap &nodes);
	void meassureNodes(nodeMap &nodes);

	int fetchData(Node &node, int length);

	bool lockW;
	bool lockRW;
	bool lockR;
	bool lockDR;

	bool updateW;
	bool updateRW;

	bool halted;
	bool confirmed;

	nodeMap nowNodesR;
	nodeMap nowNodesRW;
	nodeMap nowNodesW;
	nodeMap delayedNodesR;

	nodeMap newNodesMap;

	typeMap types;

	GPIO *gpio4;


protected:
	Delegator *pDelegator;

};

#endif
